//#define BOOST_GIL_USE_CONCEPT_CHECK 1

#include <boost/gil/image.hpp>
#include <boost/gil/extension/io/png_io.hpp>

using namespace boost::gil;

template<typename pixel_color_t>
class Mandel_fn {
private:
	pixel_color_t _in_color, _out_color;
public:

	static const int MAX_ITER = 100;

	Mandel_fn(const pixel_color_t& in_color, const pixel_color_t& out_color) :
		_in_color(in_color), _out_color(out_color) {

	}

	pixel_color_t operator()(double x, double y) {
		double t = get_num_iter(x, y);
		pixel_color_t ret;
		for(int k = 0; k < num_channels<pixel_color_t>::value; ++k) {
			ret[k] = _in_color[k] * t + _out_color[k] * (1 - t);
		}
		return ret;
	}

private:
	double get_num_iter(double x, double y) {
		point2<double> Z(0, 0);
		for (int i = 0; i < MAX_ITER; ++i) { // 100 iterations
			Z = point2<double> (Z.x * Z.x - Z.y * Z.y + x, 2 * Z.x * Z.y + y);
			if (Z.x * Z.x + Z.y * Z.y > 8) {
				return pow((double) i / MAX_ITER, 0.2);
			}
		}
		return 0;
	}
};

int main() {

//	typedef gray8_image_t image_t;
	typedef rgb8_image_t image_t;


	image_t::point_t dims(1000, 1000);

	image_t myImage(dims);
	image_t::view_t myView = view(myImage);

//	Mandel_fn<gray8_pixel_t> mandel_fn(gray8_pixel_t(256), gray8_pixel_t(0));
	Mandel_fn<rgb8_pixel_t> mandel_fn(rgb8_pixel_t(0, 0, 255), rgb8_pixel_t(0, 0, 0));

	for (int y = 0; y < dims.y; ++y) {
		for (int x = 0; x < dims.x; ++x) {
			myView(x, y) = mandel_fn((double) x / dims.x * 3 - 2, (double) y
					/ dims.y * 3 - 1.5);
		}
	}

	png_write_view("mandel3.png", myView);

	return 0;
}
